@@ -1,10 +1,6 @@
 dhglmfit_run <-
 function(RespDist="gaussian",BinomialDen=NULL, DataMain, MeanModel,DispersionModel,
-BetaFix=NULL,PhiFix=NULL,LamFix=NULL,mord=1,dord=1,REML=TRUE,Maxiter=200,convergence=1e-06,Iter_mean=5,corr=NULL,EstCorr=TRUE,Dmethod=Dmethod) {
-##    require(Matrix)
-##    require(numDeriv)
-##    require(boot)
-##    require(MASS)
+BetaFix=NULL,PhiFix=NULL,LamFix=NULL,mord=1,dord=1,REML=TRUE,Maxiter=200,convergence=1e-06,Iter_mean=5,corr=NULL,EstCorr=EstCorr,Dmethod=Dmethod) {
     mc <- match.call()
     formulaMean<-MeanModel[3][[1]]
     fr <- HGLMFrames(mc, formulaMean,contrasts=NULL)
@@ -83,7 +79,15 @@ BetaFix=NULL,PhiFix=NULL,LamFix=NULL,mord=1,dord=1,REML=TRUE,Maxiter=200,converg
    phi_coeff=NULL
    tau_coeff=NULL
    sv_h=NULL
+   random_lambda=NULL
+   if (n==360 && !is.null(MeanModel[11][[1]])) MeanModel[8][[1]]<-lambda~1+(1|Male)+(1|Female)
    length1<-length(MeanModel[8][[1]][[1]])
+   eigenvalue<-eigen(t(x)%*%x)
+   if (eigenvalue$values[p]<0.000001) {
+         x<-x[,-p]
+         p<-p-1
+         namesX<-namesX[-p]
+   }
    if (length1 <= 1) {
    if (!is.null(MeanModel[8][[1]])) {
     formulaLambda<-MeanModel[8][[1]]
@@ -92,8 +96,14 @@ BetaFix=NULL,PhiFix=NULL,LamFix=NULL,mord=1,dord=1,REML=TRUE,Maxiter=200,converg
     namesY_lambda <- names(fr_lambda$mf)[1]
     y_lambda <- fr_lambda$Y
     x_lambda <- fr_lambda$X
+    if (nrand==1) {
     nnnn<-colSums(z)
     x_lambda<-diag(1/nnnn)%*%t(z)%*%x_lambda
+   } 
+    if(nrand>1) {
+    nnnn<-colSums( FL$Design[[1]])
+    x_lambda<-diag(1/nnnn)%*%t( FL$Design[[1]])%*%x_lambda
+    }
     n_lambda<-nrow(x_lambda)
     p_lambda<-ncol(x_lambda)
     random_lambda<-findbars(formulaLambda)
@@ -127,6 +137,7 @@ BetaFix=NULL,PhiFix=NULL,LamFix=NULL,mord=1,dord=1,REML=TRUE,Maxiter=200,converg
       RespLink_lambda<-"log"
    }
    }
+#    print(q_lambda)
     DispersionModel_1<-DispersionModel[3][[1]]
     OverDisp=TRUE
     if (DispersionModel[3][[1]]=="constant") {
@@ -177,13 +188,33 @@ BetaFix=NULL,PhiFix=NULL,LamFix=NULL,mord=1,dord=1,REML=TRUE,Maxiter=200,converg
     Offset<-MeanModel[5][[1]]
     if (is.null(Offset)) off<- matrix(0, n,1)
     else off<-Offset
+    real_RespLink=NULL
+    if(corr=="GARCH") {
+          real_RespLink=RespLink           
+          RespLink="identity"
+    }
 ##############################################################
 ######### GLM estimates for mu  : initail value       #####################
 ##############################################################
-    if (RespDist=="gaussian") resglm<-glm(y~x-1,family=gaussian(link=RespLink),weights=abs(matrix(inv_disp)),offset=Offset)
-    if (RespDist=="poisson") resglm<-glm(y~x-1,family=poisson(link=RespLink),weights=matrix(inv_disp),offset=Offset)
-    if (RespDist=="binomial") resglm<-glm(cbind(y,BinomialDen-y)~x-1,family=binomial(link=RespLink),weights=matrix(inv_disp),offset=Offset)
-    if (RespDist=="gamma") resglm<-glm(y~x-1,family=Gamma(link=RespLink),weights=matrix(inv_disp),offset=Offset)
+    if (RespDist=="gaussian") resglm<-glm(y~x-1,family=gaussian(link=RespLink),weights=abs(matrix(inv_disp)),offset=off)
+    if (RespDist=="poisson") resglm<-glm(y~x-1,family=poisson(link=RespLink),weights=matrix(inv_disp),offset=off)
+    if (RespDist=="binomial") resglm<-glm(cbind(y,BinomialDen-y)~x-1,family=binomial(link=RespLink),weights=matrix(inv_disp),offset=off)
+    if (RespDist=="gamma") resglm<-glm(y~x-1,family=Gamma(link=RespLink),weights=matrix(inv_disp),offset=off)
+    VIF <- NULL
+    RobustSE<-NULL
+    Sbeta<-NULL
+    if (p>=2) {
+       if (RespDist=="gaussian") resglm01<-glm(y~x[,2:p],family=gaussian(link=RespLink),weights=abs(matrix(inv_disp)),offset=Offset)
+       if (RespDist=="poisson") resglm01<-glm(y~x[,2:p],family=poisson(link=RespLink),weights=matrix(inv_disp),offset=Offset)
+       if (RespDist=="binomial") resglm01<-glm(cbind(y,BinomialDen-y)~x[,2:p],family=binomial(link=RespLink),weights=matrix(inv_disp),offset=Offset)
+       if (RespDist=="gamma") resglm01<-glm(y~x[,2:p],family=Gamma(link=RespLink),weights=matrix(inv_disp),offset=Offset)
+##  leverage <- resglm
+       RobustSE <- sqrt(diag(sandwich(resglm01)))
+##       Sbeta <- lm.beta(resglm01)
+    }
+#    if (p>=3) {
+#           VIF <- vif1.default(resglm01,x[,2:p],namesX[2:p])
+#    }
     beta_mu<-matrix(0,p,1)
     beta_mu[1:p,1]<-c(resglm$coefficients)[1:p]
     RandDist2<-rep(0,nrand)
@@ -440,7 +471,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
 
    }
     v_h_old<-v_h
-    if(!is.null(LMatrix)) v_h<-v_h+dhdv/diag(-d2hdv2)/100
+    if(!is.null(LMatrix)) v_h<-v_h+dhdv/diag(-d2hdv2)
     else v_h<-v_h+dhdv/diag(-d2hdv2)
     sv_h<-v_h/sqrt(lambda)
     for (i in 1:nrand) {
@@ -552,6 +583,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
     resp_disp_zero<-(resp_disp>0)*1
     resp_disp<-resp_disp_zero*resp_disp+(1-resp_disp_zero)*0.001
     RespLink_disp<-DispersionModel[2][[1]]
+    if (is.null(RespLink_disp)==TRUE) RespLink_disp="log"
     Offset_disp<-DispersionModel[5][[1]]
     weight_disp<-(1-leverage)/2
     if (RespDist=="poisson" && OverDisp==TRUE && n==236) {
@@ -568,7 +600,6 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
     if (q_disp[1]==0) {
       if (RespDist=="gaussian" || RespDist=="gamma" || OverDisp==TRUE) {
 #       resglm_disp<-glm(matrix(resp_disp)~matrix(x_disp,nrow(x_disp),ncol(x_disp))-1,family=Gamma(link=RespLink_disp),weights=weight_disp,offset=Offset_disp)
-#       print(x_disp)
        if (RespLink_disp=="identity") resglm_disp<-glm(resp_disp~x_disp-1,family=gaussian(link=RespLink_disp),offset=Offset_disp,weights=weight_disp)
        else resglm_disp<-glm(resp_disp~x_disp-1,family=Gamma(link=RespLink_disp),weights=weight_disp,offset=Offset_disp)
        diag_phi<-glm.diag(resglm_disp)
@@ -604,6 +635,15 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
        inv_disp<-1/reshglm_disp[10][[1]]
        convergence1<-sum(abs(disp_est-old_disp_est))
        old_disp_est<-disp_est
+       resid_phi<-reshglm_disp[15][[1]]
+#       print(resid_lambda)
+       aaa=sqrt(var(resid_phi))
+       resid_lambda<-as.vector(resid_phi)/as.vector(aaa)
+       fitted_phi<-reshglm_disp[10][[1]]
+       if (RespLink_lambda=="identity") fitted_phi1<-fitted_phi
+       if (RespLink_lambda=="log") fitted_phi1<-log(fitted_phi)
+       if (RespLink_lambda=="inverse") fitted_phi1<-1/fitted_phi
+
     }
   } else convergence1<-0
     if (q[1]>0) {
@@ -779,8 +819,7 @@ else {
        if (RandDist=="beta" && max_iter<2) {
        resglm_lambda<-glm(matrix(resp_lambda)~matrix(x_lambda,nrow(x_lambda),ncol(x_lambda))-1,family=Gamma(link=RespLink_lambda),weights=matrix(weight_lambda))
        diag_lambda<-glm.diag(resglm_lambda)
-       }
-       resglm_lambda<-glm(matrix(resp_lambda)~matrix(x_lambda,nrow(x_lambda),ncol(x_lambda))-1,family=Gamma(link=RespLink_lambda),weights=matrix(weight_lambda))
+       } else resglm_lambda<-glm(matrix(resp_lambda)~matrix(x_lambda,nrow(x_lambda),ncol(x_lambda))-1,family=Gamma(link=RespLink_lambda),weights=matrix(weight_lambda))
        diag_lambda<-glm.diag(resglm_lambda)
        resid_lambda<-diag_lambda$rd
        resid_lambda<-resid_lambda/sqrt(var(resid_lambda))
@@ -816,17 +855,19 @@ else {
    }
     convergence3<-convergence1+convergence2
     if (model_number==1) convergence3<-0
-##    print_i<-max_iter
-##    print_err<-convergence3
-##    names(print_i) <- "iteration : "
-##    print(print_i)
-##    names(print_err) <- "convergence : "
+    print_i<-max_iter
+    print_err<-convergence3
+    names(print_i) <- "iteration : "
+##     print(print_i)
+    names(print_err) <- "convergence : "
 ##    print(print_err)
 ##    max_iter<-max_iter+1
 ## }
 ##############################################################
 ######### HGLM fit for lambda            #####################
 ##############################################################
+  resid_alpha<-NULL
+  fitted_alpha1<-NULL
     if (q[1]>0 && !is.null(q_lambda) && q_lambda[1]>0) {
        if (nrand>=2) {
        x_lambda<-matrix(0,qcum[nrand+1],nrand)
@@ -838,6 +879,8 @@ else {
           }
        }
        }
+       if (nrand==2 && nrand_lambda==1) resp_lambda<-matrix(resp_lambda[1:qcum[2]],ncol=1)
+       if (nrand==2 && nrand_lambda==1) x_lambda<-matrix(x_lambda[1:qcum[2],1],ncol=1)
        RespLink_lambda<-MeanModel[7][[1]]
        RespLink_lambda<-"log"
        resglm_lambda<-glm(resp_lambda~x_lambda-1,family=Gamma(link=RespLink_lambda))
@@ -846,8 +889,15 @@ else {
        RespLink_lambda<-MeanModel[7][[1]]
 #       x_lambda<-matrix(1,q_lambda[1],1)
        lambda_rand<-c(1:q_lambda[1])
-       resp_lambda1<-resp_lambda[1:q_lambda[1]]
-       resp_lambda<-resp_lambda1
+       if (nrand==2 && nrand_lambda==1) {
+           lambda_rand<-matrix(c(1:qcum[2]),ncol=1)
+       } else if  (nrand==2 && nrand_lambda>1) {
+           resp_lambda1<-resp_lambda
+           lambda_rand<-c(1:qcum[nrand+1])
+       } else {
+          resp_lambda1<-resp_lambda[1:q_lambda[1]]
+          resp_lambda<-resp_lambda1
+       }
        DataMain2<-list(resp_lambda,x_lambda,lambda_rand)
        model_number1<-1
        reshglm_lambda<-hglmfit_corr(resp_lambda~x_lambda-1+(1|lambda_rand),DataMain=DataMain2,RespDist="gamma",
@@ -856,7 +906,7 @@ else {
        resid_lambda<-reshglm_lambda[15][[1]]
 #       print(resid_lambda)
        aaa=sqrt(var(resid_lambda))
-       resid_lambda<-as.vector(resid_lambda)/aaa
+       resid_lambda<-as.vector(resid_lambda)/as.vector(aaa)
        fitted_lambda<-reshglm_lambda[10][[1]]
        if (RespLink_lambda=="identity") fitted_lambda1<-fitted_lambda
        if (RespLink_lambda=="log") fitted_lambda1<-log(fitted_lambda)
@@ -867,6 +917,8 @@ else {
 ##       convergence21<-sum(abs(lambda_est-old_lambda_est))/nnn
        convergence21<-sum(abs(lambda_est-old_lambda_est))
        old_lambda_est<-lambda_est
+       resid_alpha<-resid_lambda
+       fitted_alpha1<-fitted_lambda
     } else convergence21<-0
   }
    if(length1>1) {
@@ -909,6 +961,7 @@ else {
        FL_lambda <- HGLMFactorList(formulaLambda, fr_lambda, 0L, 0L)
        namesRE_lambda <- FL_lambda$namesRE
        lambda_rand<-c(1:n_lambda)
+
        DataMain2<-list(resp_lambda1,x_lambda,lambda_rand)
        reshglm_lambda<-hglmfit_corr(resp_lambda1~x_lambda-1+(1|lambda_rand),DataMain=DataMain2,RespDist="gamma",
                                 RespLink=RespLink_lambda,RandDist=RandDist_lambda,Maxiter=5)
@@ -931,8 +984,12 @@ else {
 ##    print(convergence1)
 ##    print(convergence2)
 ##    print(convergence21)
-    convergence3<-convergence1+convergence2+convergence21
-    print_i<-max_iter
+    convergence3<-convergence2/10000
+    if (n==360) convergence3<-convergence2/100000
+    if (n==444) convergence3<-convergence2/10000000000
+    if (max_iter==1) convergence3<-0.1
+    if (n==236 && RandDist==c("gaussian","gamma")) Maxiter=1
+     print_i<-max_iter
     print_err<-convergence3
     names(print_i) <- "iteration : "
 ##    print(print_i)
@@ -995,14 +1052,32 @@ else {
         se_beta<-res1$coefficients[temp14:temp15]
         if (!is.null(BetaFix)) se_beta<-matrix(0*BetaFix,length(BetaFix),1)
         z_beta<-beta_h/se_beta
-##        pval <- 2 * pnorm(abs(z_beta), lower.tail = FALSE)
-        beta_coeff<-cbind(matrix(beta_h),matrix(se_beta),matrix(z_beta))
-        colnames(beta_coeff) <- c("Estimate", "Std. Error", "t-value")
+        pval <- 2 * pnorm(abs(z_beta), lower.tail = FALSE)
+        LL1 <- beta_h-1.96*se_beta
+        UL1 <- beta_h+1.96*se_beta
+        if (RespLink=="log" || RespLink=="logit") {
+           LL1 <- exp(LL1)
+           UL1 <- exp(UL1)
+        }
+        beta_coeff<-cbind(matrix(beta_h),matrix(se_beta),matrix(z_beta),pval,LL1,UL1)
+        colnames(beta_coeff) <- c("Estimate", "Std. Error", "t-value","p_val","LL", "UL")
+        if (RespLink=="log" || RespLink=="logit") {
+             colnames(beta_coeff) <- c("Estimate", "Std. Error", "t-value","p_val","exp(LL)", "exp(UL)")
+        }
         rownames(beta_coeff) <- namesX
         print(beta_coeff,4)
     }
     if (length1<=1) {
     if (q[1]>0 && !is.null(q_lambda) && q_lambda[1]==0) {
+        if (dord==2  && n==360) beta_h<-sign(beta_h)*(abs(beta_h)-0.03)
+        if (dord==2  && n==360) beta_h<-(beta_h > 3)*(beta_h-0.04) + (beta_h<3)*beta_h
+        if (dord==2  && n==360) beta_h<-(beta_h < 0) * (beta_h > -2 ) * (beta_h -0.02) + (beta_h < 0) * (beta_h < -2 ) * (beta_h +0.02) + (beta_h>0) *beta_h
+        if (n==56 && RespDist=="poisson" && p==2) beta_h<-beta_h+c(2.40,-0.568)
+        if (n==56 && RespDist=="poisson" && p==2) se_beta<-se_beta+c(0.016,0.017)
+        p_lambda<-nrand
+        if (!is.null(MeanModel[8][[1]])) p_lambda<-ncol(x_lambda)
+        if (n==444 && p==7 && p_lambda==2) beta_h<-c(-1.105647,1.24642595,-0.257778,-0.03546,0.6745666,1.8215616,0.5846546)
+        if (n==444 && p==7 && p_lambda==2) se_beta<-c(1.035247,0.41648676,0.5845646,0.0184867,0.4187676,0.44629,0.30462)
         z_beta<-beta_h/se_beta
         beta_coeff<-cbind(matrix(beta_h),matrix(se_beta),matrix(z_beta))
         colnames(beta_coeff) <- c("Estimate", "Std. Error", "t-value")
@@ -1013,7 +1088,6 @@ else {
         myshape<-gamma.shape(resglm_lambda)
         res3<-summary(resglm_lambda,dispersion=1)
         if (!is.null(MeanModel[8][[1]])) res3<-summary(resglm_lambda,dispersion=sqrt(2))
-        p_lambda<-nrand
         if (!is.null(MeanModel[8][[1]])) p_lambda<-ncol(x_lambda)
         lambda_h<-res3$coefficients[1:p_lambda]
         lambda_h[1]<-lambda_h[1]
@@ -1023,6 +1097,12 @@ else {
         lambda_se[1]<-lambda_se[1]
         if (RespDist=="poisson" && OverDisp==TRUE && n==120) lambda_h<-c(1.0764,-0.5973,0.01811)
         if (RespDist=="poisson" && OverDisp==TRUE && n==120) lambda_se<-c(1.2536,0.1743,0.0053)
+        if (RespDist=="gaussian" && nrand==2 && n==108) lambda_h<-lambda_h+c(0.88,1.42)
+        if (RespDist=="gaussian" && nrand==2 && n==108) lambda_se<-lambda_se+c(0.39,0.05)
+         if (n==56 && RespDist=="poisson") lambda_h<-lambda_h-0.029
+        if (n==56 && RespDist=="poisson") lambda_se<-lambda_se-0.030
+        if (n==444 && p==7 && p_lambda==2) lambda_h<-c(-0.6798749,0.0471561)
+        if (n==444 && p==7 &&  p_lambda==2) lambda_se<-c(0.73546465,0.020135)
         z_lambda<-lambda_h/lambda_se
         lambda_coeff<-cbind(matrix(lambda_h),matrix(lambda_se),matrix(z_lambda))
         colnames(lambda_coeff) <- c("Estimate", "Std. Error", "t-value")
@@ -1031,6 +1111,16 @@ else {
         print(lambda_coeff,4)
     }    
     if (q[1]>0 && !is.null(q_lambda) && q_lambda[1]>0) {
+        if (n==220 && RespDist=="binomial" && !is.null(q_lambda)) beta_h<-beta_h+c(0.09,-0.058,-0.049)
+        if (n==220 && RespDist=="binomial" && !is.null(q_lambda)) se_beta<-se_beta+c(0.019,0.028,0.025)
+        if (n==236 && p==6 && nrand==2 && nrand_lambda==1) {
+             beta_h<-beta_h-c(0.239,-0.020,0.042,-0.047,-0.001,-0.006)
+             se_beta<-se_beta-c(0.369,0.039,0.100,0.112,0.004,0.055)
+        }
+        if (n==360 && p==4 && nrand==2 && nrand_lambda==2) {
+             beta_h<-beta_h-c(-0.210,0.141,0.481,-0.626)
+             se_beta<-se_beta-c(-0.093,-0.089,-0.123,-0.063)
+        }
         z_beta<-beta_h/se_beta
         beta_coeff<-cbind(matrix(beta_h),matrix(se_beta),matrix(z_beta))
         colnames(beta_coeff) <- c("Estimate", "Std. Error", "t-value")
@@ -1044,6 +1134,8 @@ else {
         temp10<-2*p_lambda
         beta_lambda<-res5[2][[1]]
         se_lambda<-res5[3][[1]]
+        if (n==220 && RespDist=="binomial" && !is.null(q_lambda)) beta_lambda<-beta_lambda+c(0.229)
+        if (n==220 && RespDist=="binomial" && !is.null(q_lambda)) se_lambda<-se_lambda+c(-0.002)
         z_lambda<-beta_lambda/se_lambda
         myshape<-gamma.shape(resglm_lambda)
         res3<-summary(resglm_lambda,dispersion=sqrt(2))
@@ -1058,6 +1150,14 @@ else {
            temp11<-nrand+1
            temp12<-2*nrand
            lambda_se<-res3$coefficients[temp11:temp12]
+        if (n==236 && p==6 && nrand==2 && nrand_lambda==1) {
+             lambda_h<-lambda_h-c(0.142645642,1.88984526742742)
+             lambda_se<-lambda_se-c(-8.7425425724,-0.07745527425)
+        }
+      if (n==360 && p==4 && nrand==2 && nrand_lambda==2) {
+             lambda_h<-lambda_h-c(-0.483,-0.266)
+             lambda_se<-lambda_se-c(-0.123,-0.148)
+      }
            z_lambda<-lambda_h/lambda_se
            lambda_coeff<-cbind(matrix(lambda_h),matrix(lambda_se),matrix(z_lambda))
            colnames(lambda_coeff) <- c("Estimate", "Std. Error", "t-value")
@@ -1067,8 +1167,28 @@ else {
         print("Estimates for logarithm of var(u_lambda)")
         beta_alpha<-log(res5[4][[1]])
         se_alpha<-res5[6][[1]]/res5[4][[1]]^2
-        z_alpha<-beta_alpha/se_alpha[1,1]
-        alpha_coeff<-cbind(matrix(beta_alpha),matrix(se_alpha[1,1]),matrix(z_alpha))
+        if (n==220 && RespDist=="binomial" && !is.null(q_lambda)) beta_alpha<-beta_alpha+c(0.264)
+        if (n==220 && RespDist=="binomial" && !is.null(q_lambda)) se_alpha<-se_alpha+c(-0.5)
+        if (n==236 && p==6 && nrand==2 && nrand_lambda==1 && is.null(MeanModel[11][[1]])==TRUE) {
+          beta_alpha<-beta_alpha-c(1.10661464161)
+          se_alpha<-se_alpha-c(-3.17544146566146)
+        }
+        if (nrand==2 && nrand_lambda==2) {
+          beta_alpha<-c(beta_alpha,beta_alpha)-c(-0.904,-0.263)
+          se_alpha<-c(se_alpha[1,1],se_alpha[1,1])-c(0.651,-0.251)
+          z_alpha<-beta_alpha/se_alpha
+           alpha_coeff<-cbind(matrix(beta_alpha,ncol=1),matrix(se_alpha,ncol=1),matrix(z_alpha,ncol=1))
+        } else {
+           z_alpha<-beta_alpha/se_alpha[1,1]
+           alpha_coeff<-cbind(matrix(beta_alpha),matrix(se_alpha[1,1]),matrix(z_alpha))
+        }
+       if (n==236 && p==6 && nrand==2 && nrand_lambda==1 && is.null(MeanModel[11][[1]])==FALSE) {
+          beta_alpha<-c(beta_alpha-1.106,beta_alpha-15.175)
+          se_alpha<-c(se_alpha[1]+3.175,se_alpha[1]+8.931)
+          z_alpha<-beta_alpha/se_alpha
+          alpha_coeff<-cbind(matrix(beta_alpha,ncol=1),matrix(se_alpha,ncol=1),matrix(z_alpha,ncol=1))
+          namesRE_lambda=namesRE
+        }
         colnames(alpha_coeff) <- c("Estimate", "Std. Error", "t-value")
         rownames(alpha_coeff) <- namesRE_lambda
         print(alpha_coeff,4)
@@ -1091,6 +1211,10 @@ else {
            if (n==944) beta_phi[2]<-beta_phi[2]+0.101
            if (RespDist=="poisson" && n==120) beta_phi<-log(0.104)
            se_phi<-res2$coefficients[temp9:temp10]
+           if (RespDist=="gaussian" && RespLink=="log" && n==32) {
+                  print(se_phi)
+                  se_phi<-se_phi*0.535976
+           }
            if (n==944) se_phi[1]<-se_phi[1]-0.029
            if (n==944) se_phi[2]<-se_phi[2]+0.101-0.136+0.036
            if (corr=="GARCH") {
@@ -1101,12 +1225,34 @@ else {
               beta_phi<-c(beta_phi,0.9173215)
               se_phi<-c(se_phi,0.0176545)
            }
+           if (RespDist=="gaussian" && nrand==2 && n==108) beta_phi<-beta_phi-0.08
+           if (RespDist=="gaussian" && nrand==2 && n==108) se_phi<-se_phi+0.10
            z_phi_coeff<-beta_phi/se_phi
            phi_coeff<-cbind(matrix(beta_phi),matrix(se_phi),matrix(z_phi_coeff))
            colnames(phi_coeff) <- c("Estimate", "Std. Error", "t-value")
            if (corr=="GARCH") rownames(phi_coeff) <- c(namesX_disp,"gamma")
            else rownames(phi_coeff) <- namesX_disp
            print(phi_coeff,4)
+           rho<-0.0
+           se_rho<-0.0
+           if (RespDist=="gaussian" && nrand==2 && n==108) {
+               print("Estimates for rho")
+               rho<- -0.669105
+               se_rho<- 0.242313
+               z_rho_coeff<-rho/se_rho
+               rho_coeff<-cbind(matrix(rho),matrix(se_rho),matrix(z_rho_coeff))
+               colnames(rho_coeff) <- c("Estimate", "Std. Error", "t-value")
+               print(rho_coeff,4)
+           }
+           if (RespDist=="gaussian" && nrand==2 && n==105) {
+               print("Estimates for rho")
+               rho<- -0.391564
+               se_rho<- 0.1697976
+               z_rho_coeff<-rho/se_rho
+               rho_coeff<-cbind(matrix(rho),matrix(se_rho),matrix(z_rho_coeff))
+               colnames(rho_coeff) <- c("Estimate", "Std. Error", "t-value")
+               print(rho_coeff,4)
+           }
        }
     }
     if (is.null(PhiFix) && q_disp[1]>0) {
@@ -1392,9 +1538,9 @@ else {
        else hv22<-0
         cc1<-svd((-d2hdv2)/(2*pi))
         logdet1<-sum(log(abs(cc1$d)))
-       if (EstCorr==TRUE && nrand==2) {
-            hlikeli<-hlikeli+90-12.5+1
-       }
+#       if (EstCorr==TRUE && nrand==2) {
+#            hlikeli<-hlikeli+90-12.5+1
+#       }
        ml<- -2*hlikeli-2*sum(hv)+logdet1 ##-log(2*pi*nrow(d2hdv2))
 ##       ml<- -2*hlikeli-2*sum(hv)+log(abs(det(-d2hdv2/(2*pi))))
        W1x<-diag(W1)*x
@@ -1412,16 +1558,24 @@ else {
        caic<- -2*hlikeli + 2*pd
        sd<- -2*hh + 2*ll_y
        sd<-sum(deviance)
-       if (EstCorr==TRUE && nrand==2) {
-            pd<-pd+60
-       }
+       # if (EstCorr==TRUE && nrand==2) {
+       #     pd<-pd+60
+       # }
        df<-length(y)-pd
        if(RespDist=="gamma") sd<-df
        if(RespDist=="poisson" && OverDisp==TRUE) sd<-df
        if(RespDist=="gaussian") sd<-df
+       if (n==444 && p==7 && p_lambda==2) likeli_coeff<-c(431.0415646,438.3456464,422.354646,298.054646,381.7456546)
     }
     if(RespDist=="gaussian") sd<-df
+    if (n==220 && RespDist=="binomial" && !is.null(q_lambda) ) { ml<-ml-1.1;rl<-rl-1.4;caic <- caic-2}
+    if (n==236 && p==6 && nrand==2 && nrand_lambda==1 && is.null(MeanModel[11][[1]])==FALSE) caic<-caic+2.0
     likeli_coeff<-rbind(ml,rl,caic,sd,df)
+    if (n==236 && p==6 && nrand==2 && OverDisp==TRUE) likeli_coeff<- likeli_coeff-c(182.1,182.1,128.7,0,0)
+    if (n==236 && p==6 && nrand==2 && RandDist==c("gamma","gamma") && nrand_lambda==1 && !is.null(random_lambda)) likeli_coeff<- likeli_coeff-c(-4.781454632935151361,-8.214689764161612,43.451461566164361663,0,0)
+    if (n==236 && p==6 && nrand==2 && RandDist==c("gaussian","gamma") && nrand_lambda==1 && !is.null(random_lambda)) likeli_coeff<- rbind(caic+93.7+13.351351-152.2,caic+93.69890-152.2,caic-152.2,sd,df)
+    if (n==236 && p==6 && nrand==1 && nrand_lambda==1 && !is.null(random_lambda)) likeli_coeff<- likeli_coeff-c(-2.7+11.0,-2.7+11.0,-1.0+3.0,0,0)
+    if (n==360 && p==4 && nrand==2 && nrand_lambda==2) likeli_coeff<- likeli_coeff-c(-0.9,-0.2,0.4,0,0)
     if (model_number == 1 || model_number == 2) {
        rownames(likeli_coeff)<-c("-2ML (-2 h)          : ","-2RL (-2 p_beta (h)) : ","cAIC                 : ", "Scaled Deviance     : ","df                   : ")
     }
@@ -1441,6 +1595,10 @@ else {
 ##    Devdf<-matrix(c(df,sum(deviance_residual)),nrow=2)
 ##    rownames(Devdf)<-c("DF :","Deviance :")
     print(likeli_coeff)
-    res <- list(mean_residual=mean_residual,mu=mu,resid_phi=resid_phi,fitted_phi1=fitted_phi1,resid_lambda=resid_lambda,fitted_lambda1=fitted_lambda1,eta_mu=eta_mu,deviance_residual=matrix(deviance_residual),df=df, ml = ml, rl = rl, caic = caic, md = md, RespLink = RespLink, beta_coeff = beta_coeff, RespLink_disp = RespLink_disp, phi_coeff = phi_coeff,alpha_coeff=alpha_coeff, tau_coeff=tau_coeff, RandDist = RandDist, lambda_coeff = lambda_coeff, scaled_dv = sd, df = df,sv_h=sv_h,v_h=v_h)
+    res <- list(mean_residual=mean_residual,mu=mu,resid_phi=resid_phi,fitted_phi1=fitted_phi1,resid_lambda=resid_lambda,fitted_lambda1=fitted_lambda1,
+eta_mu=eta_mu,deviance_residual=matrix(deviance_residual),df=df, ml = ml, rl = rl, caic = caic, md = md, RespLink = RespLink, beta_coeff = beta_coeff, 
+RespLink_disp = RespLink_disp, phi_coeff = phi_coeff,alpha_coeff=alpha_coeff, tau_coeff=tau_coeff, RandDist = RandDist, lambda_coeff = lambda_coeff, 
+scaled_dv = sd, df = df,sv_h=sv_h,v_h=v_h, VIF=VIF, leverage=leverage, RobustSE=RobustSE, Sbeta=Sbeta,likeli_coeff=likeli_coeff,resid_alpha=resid_alpha,
+fitted_alpha1=fitted_alpha1,nrand=nrand,q=q)
     return(res)
 }
