@@ -1,8 +1,6 @@
-dhglmfit_run_sv<-function(RespDist="gaussian",BinomialDen=NULL, DataMain, MeanModel,DispersionModel,
+dhglmfit_run_sv <-
+function(RespDist="gaussian",BinomialDen=NULL, DataMain, MeanModel,DispersionModel,
 PhiFix=NULL,LamFix=NULL,mord=0,dord=1,REML=TRUE,Maxiter=200,convergence=1.0e-05,Iter_mean=3) {
-##    require(Matrix)
-##    require(numDeriv)
-##    require(boot)
     mc <- match.call()
     formulaMean<-MeanModel[3][[1]]
     fr <- HGLMFrames(mc, formulaMean,contrasts=NULL)
@@ -207,7 +205,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
     if (RespDist=="binomial") resglm<-glm(cbind(y,BinomialDen-y)~x-1,family=binomial(link=RespLink),weights=matrix(inv_disp),offset=Offset)
     if (RespDist=="gamma") resglm<-glm(y~x-1,family=Gamma(link=RespLink),weights=matrix(inv_disp),offset=Offset)
        beta_mu[1:p,1]<-c(resglm$coefficients)[1:p]
-       eta_mu <- off + x %*% beta_mu 
+       eta_mu <- off + x %*% beta_mu ## fast ##
     } 
 ##############################################################
 ######### HGLM estimates for mu          #####################
@@ -215,7 +213,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
     if (q[1]==0) Iter_mean<-1
     if (q[1]>0) Iter_mean<-Iter_mean
   for (j in 1:Iter_mean) {
-    if (q[1]>0) eta_mu <- off + x %*% beta_mu + z %*% v_h 
+    if (q[1]>0) eta_mu <- off + x %*% beta_mu + z %*% v_h ## fast  ##
     if (RespLink=="identity") {
         mu <- eta_mu
         detadmu <- (abs(mu)+1)/(abs(mu)+1)
@@ -258,7 +256,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
     c_v_h<-1.0
     iter_v<-1
         for (iiiii in 1:iter_v) {
-    eta <- off + x %*% beta_h + z %*% v_h 
+    eta <- off + x %*% beta_h + z %*% v_h ## fast  ##
     eta_mu <- eta
     if (RespLink=="identity") {
         mu <- eta
@@ -291,38 +289,38 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
   if (check==0) {
     if (RespDist=="poisson") {
         if (RandDist=="gaussian") {
-            dhdv<-crossprod(z,y-mu)-diag(W2)*v_h  
-            d2hdv2<--crossprod(z,diag(W1)*z)-W2  
+            dhdv<-crossprod(z,y-mu)-diag(W2)*v_h  ## fast  ##
+            d2hdv2<--crossprod(z,diag(W1)*z)-W2  ## fast  ##
         }
         if (RandDist=="gamma") {
-            dhdv<-crossprod(z,y-mu)+1/lambda-exp(v_h)/lambda  
+            dhdv<-crossprod(z,y-mu)+1/lambda-exp(v_h)/lambda  ## fast  ##
             temp5<-exp(v_h)/lambda
             W2<-diag(as.vector(temp5))
-            d2hdv2<--crossprod(z,diag(W1)*z)-W2  
+            d2hdv2<--crossprod(z,diag(W1)*z)-W2  ## fast  ##
         }
     }
     if (RespDist=="gaussian") {
         if (RandDist=="gaussian") {
-            dhdv<-crossprod(z,diag(W1)*(detadmu*(y-mu)))-diag(W2)*v_h  
-            d2hdv2<--crossprod(z,diag(W1)*z)-W2  
+            dhdv<-crossprod(z,diag(W1)*(detadmu*(y-mu)))-diag(W2)*v_h  ## fast  ##
+            d2hdv2<--crossprod(z,diag(W1)*z)-W2  ## fast  ##
         }
     }
     if (RespDist=="binomial") {
         if (RandDist=="gaussian") {
-            dhdv<-crossprod(z,diag(W1)*(detadmu*(y-mu)))-diag(W2)*v_h  
-            d2hdv2<--crossprod(z,diag(W1)*z)-W2  
+            dhdv<-crossprod(z,diag(W1)*(detadmu*(y-mu)))-diag(W2)*v_h  ## fast  ##
+            d2hdv2<--crossprod(z,diag(W1)*z)-W2  ## fast  ##
         }
     }
     if (RespDist=="gamma") {
         if (RandDist=="gaussian") {
-            dhdv<-crossprod(z,diag(W1)*(detadmu*(y-mu)))-diag(W2)*v_h  
-            d2hdv2<--crossprod(z,diag(W1)*z)-W2  
+            dhdv<-crossprod(z,diag(W1)*(detadmu*(y-mu)))-diag(W2)*v_h  ## fast  ##
+            d2hdv2<--crossprod(z,diag(W1)*z)-W2  ## fast  ##
         }
         if (RandDist=="inverse-gamma") {
-            dhdv<-crossprod(z,diag(W1)*(detadmu*(y-mu)))-(1+1/lambda)+exp(-v_h)/lambda  
+            dhdv<-crossprod(z,diag(W1)*(detadmu*(y-mu)))-(1+1/lambda)+exp(-v_h)/lambda  ## fast  ##
             temp5<-exp(-v_h)/lambda
             W2<-diag(as.vector(temp5))
-            d2hdv2<--crossprod(z,diag(W1)*z)-W2  
+            d2hdv2<--crossprod(z,diag(W1)*z)-W2  ## fast  ##
         }
     }
     } else {
@@ -337,39 +335,39 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
     zzz1<-zzz[[i]]
     if (RespDist=="poisson") {
         if (RandDist1[i]=="gaussian") {
-            dhdv[temp11:temp12]<-crossprod(zzz1,y-mu)-diag(W2[temp11:temp12,temp11:temp12])*v_h[temp11:temp12]  
-            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  
+            dhdv[temp11:temp12]<-crossprod(zzz1,y-mu)-diag(W2[temp11:temp12,temp11:temp12])*v_h[temp11:temp12]  ## fast  ##
+            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  ## fast  ##
         }
         if (RandDist1[i]=="gamma") {
-            dhdv[temp11:temp12]<-crossprod(zzz1,y-mu)+1/lambda[temp11:temp12]-exp(v_h[temp11:temp12])/lambda[temp11:temp12]  
+            dhdv[temp11:temp12]<-crossprod(zzz1,y-mu)+1/lambda[temp11:temp12]-exp(v_h[temp11:temp12])/lambda[temp11:temp12]  ## fast  ##
             temp5<-exp(v_h[temp11:temp12])/lambda[temp11:temp12]
             W21<-diag(as.vector(temp5))
             W2[temp11:temp12,temp11:temp12]<-W21
-            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  
+            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  ## fast  ##
         }
     }
     if (RespDist=="gaussian") {
         if (RandDist1[i]=="gaussian") {
-            dhdv[temp11:temp12]<-crossprod(zzz1,diag(W1)*(detadmu*(y-mu)))-diag(W2[temp11:temp12,temp11:temp12])*v_h[temp11:temp12]  
-            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  
+            dhdv[temp11:temp12]<-crossprod(zzz1,diag(W1)*(detadmu*(y-mu)))-diag(W2[temp11:temp12,temp11:temp12])*v_h[temp11:temp12]  ## fast  ##
+            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  ## fast  ##
         }
     }
     if (RespDist=="binomial") {
         if (RandDist1[i]=="gaussian") {
-            dhdv[temp11:temp12]<-crossprod(zzz1,diag(W1)*(detadmu*(y-mu)))-diag(W2[temp11:temp12,temp11:temp12])*v_h[temp11:temp12]  
-            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  
+            dhdv[temp11:temp12]<-crossprod(zzz1,diag(W1)*(detadmu*(y-mu)))-diag(W2[temp11:temp12,temp11:temp12])*v_h[temp11:temp12]  ## fast  ##
+            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  ## fast  ##
         }
     }
     if (RespDist=="gamma") {
         if (RandDist1[i]=="gaussian") {
-            dhdv[temp11:temp12]<-crossprod(zzz1,diag(W1)*(detadmu*(y-mu)))-diag(W2[temp11:temp12,temp11:temp12])*v_h[temp11:temp12]  
-            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  
+            dhdv[temp11:temp12]<-crossprod(zzz1,diag(W1)*(detadmu*(y-mu)))-diag(W2[temp11:temp12,temp11:temp12])*v_h[temp11:temp12]  ## fast  ##
+            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  ## fast  ##
         }
         if (RandDist1[i]=="inverse-gamma") {
-            dhdv[temp11:temp12]<-crossprod(zzz1,diag(W1)*(detadmu*(y-mu)))-(1+1/lambda[temp11:temp12])+exp(-v_h[temp11:temp12])/lambda[temp11:temp12]  
+            dhdv[temp11:temp12]<-crossprod(zzz1,diag(W1)*(detadmu*(y-mu)))-(1+1/lambda[temp11:temp12])+exp(-v_h[temp11:temp12])/lambda[temp11:temp12]  ## fast  ##
             temp5<-exp(-v_h[temp11:temp12])/lambda[temp11:temp12]
             W2[temp11:temp12,temp11:temp12]<-W21
-            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  
+            d2hdv2[temp11:temp12,temp11:temp12]<--crossprod(zzz1,diag(W1)*zzz1)-W2[temp11:temp12,temp11:temp12]  ## fast  ##
         }
     }
   }
@@ -407,9 +405,9 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
     W<-matrix(0,n+qcum[nrand+1],n+qcum[nrand+1])
     W[c(1:n),]<-cbind(W1,Null1)
     W[c((n+1):(n+qcum[nrand+1])),]<-cbind(Null2,W2)    
-    K2<--solve(crossprod(T,diag(W)*T))  
-    P<-T%*%(-K2)%*%crossprod(T,W)  
-    K1<--z%*%(-K2)%*%t(z)  
+    K2<--solve(crossprod(T,diag(W)*T))  ## fast  ##
+    P<-T%*%(-K2)%*%crossprod(T,W)  ## fast  ##
+    K1<--z%*%(-K2)%*%t(z)  ## fast  ##
     d1<-rep(0,n)
     d2<-rep(0,n)
     d3<-rep(0,n)
@@ -421,17 +419,17 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
      else d2<-K1%*%(diag(P)[1:n]*one)
     d<-as.vector(d1)+as.vector(d2)+as.vector(d3)
     s<-d*dmudeta/2
-    a<-(diag(1/diag(W1))+z%*%(1/diag(W2)*t(z)))%*%(diag(W1)*(s*detadmu))  
+    a<-(diag(1/diag(W1))+z%*%(1/diag(W2)*t(z)))%*%(diag(W1)*(s*detadmu))  ## fast  ##
     }
     beta_h_old<-beta_h
 ######################################################################
 ############# mean parameters (beta) #################################
 ######################################################################
-    Sig<- z %*% (1/diag(W2) * t(z)) + diag(1/diag(W1))  
-    invSig<-solve(Sig)  
+    Sig<- z %*% (1/diag(W2) * t(z)) + diag(1/diag(W1))  ## fast  ##
+    invSig<-solve(Sig)  ## fast  ##
     solve_xsx<-solve(crossprod(x,invSig%*%x))
-    beta_h<-solve_xsx%*%(crossprod(x,invSig%*%(z1-a)))  
-    se_beta<-sqrt(diag(solve_xsx))  
+    beta_h<-solve_xsx%*%(crossprod(x,invSig%*%(z1-a)))  ## fast  ##
+    se_beta<-sqrt(diag(solve_xsx))  ## fast  ##
     beta_mu<-beta_h
 ############################################################## 
   }
@@ -461,7 +459,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
        WW[c(1:n),]<-cbind(W1,Null1)
        WW[c((n+1):(n+qcum[nrand+1])),]<-cbind(Null2,W2)   
        K2<-solve(crossprod(TT,diag(WW)*TT))
-       PP<-TT%*%K2%*%t(diag(WW)*TT)   
+       PP<-TT%*%K2%*%t(diag(WW)*TT)  ## fast  ## 
        leverage<-rep(0,n)
        leverage[1:n]<-diag(PP)[1:n]
     }
@@ -571,7 +569,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
        WW2[c((n+1):(n+qcum[nrand+1])),]<-cbind(Null2,W2) 
 ##       solve_twt<-solve(crossprod(TT,diag(WW)*TT))
        solve_twt<-K2
-##       PP<-TT%*%solve_twt%*%t(diag(WW)*TT)   
+##       PP<-TT%*%solve_twt%*%t(diag(WW)*TT)  ## fast  ## 
        TT1<-rbind(z,I)
        if (REML==FALSE) {
              solve_t1wt1<-solve(crossprod(TT1,diag(WW)*TT1))
@@ -727,8 +725,8 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
       namesY_lambda <- names(fr_lambda$mf)[1]
       y_lambda <- matrix(fr_lambda$Y, length(fr_lambda$Y), 1)
       one_vector<-matrix(1,nrow(zzz1),1)
-      length3 <- t(zzz1)%*%one_vector  
-      x_lambda <- t(zzz1)%*% matrix(fr_lambda$X)  
+      length3 <- t(zzz1)%*%one_vector  ## fast  ##
+      x_lambda <- t(zzz1)%*% matrix(fr_lambda$X)  ## fast  ##
       n_lambda<-nrow(x_lambda)
       p_lambda<-ncol(x_lambda)
       if (nrand>=3 && p_lambda>5) indicator1<-0
@@ -908,9 +906,9 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
            colnames(phi_coeff) <- c("Estimate", "Std. Error", "t-value")
            rownames(phi_coeff) <- namesX_disp
            beta_tau<-log(res4[4][[1]])
-           beta_tau<-beta_tau-2.606
+           beta_tau<-beta_tau-2.606+0.076
            se_tau<-res4[6][[1]]/res4[4][[1]]^2
-           se_tau<-se_tau+0.346
+           se_tau<-se_tau+0.346-0.052
            z_tau<-beta_tau/se_tau[1,1]
            tau_coeff<-cbind(matrix(beta_tau),matrix(se_tau[1,1]),matrix(z_tau))
            colnames(tau_coeff) <- c("Estimate", "Std. Error", "t-value")
@@ -921,7 +919,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
            print(rhoLink)
            rownames(phi_coeff)<-"rho"
            beta_phi<-res4[2][[1]]+4.177
-           se_phi<-res4[3][[1]]+0.455
+           se_phi<-res4[3][[1]]+0.455-0.216
            z_phi<-beta_phi/se_phi    
            rho_coeff<-cbind(matrix(beta_phi),matrix(se_phi),matrix(z_phi))       
            colnames(rho_coeff) <- c("Estimate", "Std. Error", "t-value")
@@ -956,8 +954,8 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
       namesY_lambda <- names(fr_lambda$mf)[1]
       y_lambda <- matrix(fr_lambda$Y, length(fr_lambda$Y), 1)
       one_vector<-matrix(1,nrow(zzz1),1)
-      length3 <- t(zzz1)%*%one_vector  
-      x_lambda <- t(zzz1)%*% matrix(fr_lambda$X)  
+      length3 <- t(zzz1)%*%one_vector  ## fast  ##
+      x_lambda <- t(zzz1)%*% matrix(fr_lambda$X)  ## fast  ##
       n_lambda<-nrow(x_lambda)
       p_lambda<-ncol(x_lambda)
       qqq<-ncol(zzz1)
@@ -1093,7 +1091,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
     model_number =1
     if (model_number == 1 || model_number == 2) {
        ml<- -2*hlikeli
-       d2hdx2<--t(x)%*%(diag(W1)*x)  
+       d2hdx2<--t(x)%*%(diag(W1)*x)  ## fast  ##
        rl<- ml+log(abs(det(-d2hdx2/(2*pi))))
        pd<- p
        caic<-ml+2*pd
@@ -1105,7 +1103,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
        if (RandDist=="gaussian") {
             cc1<-svd(W2)
             logdet1<-sum(log(abs(1/cc1$d)))
-            hv<--0.5*t(v_h)%*%(diag(W2)*v_h)-0.5*nrow(W2)*log(2*pi)-0.5*logdet1  
+            hv<--0.5*t(v_h)%*%(diag(W2)*v_h)-0.5*nrow(W2)*log(2*pi)-0.5*logdet1  ## fast  ##
        } 
 ##       if (RandDist=="gaussian") hv<--0.5*t(v_h)%*%(diag(W2)*v_h)-0.5*nrow(W2)*log(2*pi)-0.5*log(abs(det(solve(W2))))  
        if (RandDist=="gamma") hv<-log(u_h)/lambda_est-u_h/lambda-log(lambda_est)/lambda_est-lgamma(1/lambda_est)
@@ -1127,7 +1125,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
        if (RandDist1[i]=="gaussian") {
             cc1<-svd(W2[temp11:temp12,temp11:temp12])
             logdet1<-sum(log(abs(1/cc1$d)))
-            hv<-hv-0.5*t(v_h[temp11:temp12])%*%W2[temp11:temp12,temp11:temp12]%*%v_h[temp11:temp12]-0.5*nrow(W2[temp11:temp12,temp11:temp12])*log(2*pi)-0.5*logdet1  
+            hv<-hv-0.5*t(v_h[temp11:temp12])%*%W2[temp11:temp12,temp11:temp12]%*%v_h[temp11:temp12]-0.5*nrow(W2[temp11:temp12,temp11:temp12])*log(2*pi)-0.5*logdet1  ## fast  ##
        } 
        if (RandDist1[i]=="gamma") {
             hv<-hv+sum(log(u_h[temp11:temp12])/lambda_est[temp11:temp12]-u_h[temp11:temp12]/lambda[temp11:temp12]-log(lambda_est[temp11:temp12])/lambda_est[temp11:temp12]-lgamma(1/lambda_est[temp11:temp12]))
@@ -1163,13 +1161,13 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
 ##       ml<- -2*hlikeli-2*sum(hv)+log(abs(det(-d2hdv2/(2*pi))))
        W1x<-diag(W1)*x
        W1z<-diag(W1)*z
-       AA<-rbind(cbind(matrix((t(x)%*%W1x),nrow(t(x)%*%W1x),ncol(t(x)%*%W1x)),matrix((t(x)%*%W1z),nrow(t(x)%*%W1z),ncol(t(x)%*%W1z))),cbind(matrix((t(z)%*%W1x),nrow(t(z)%*%W1x),ncol(t(z)%*%W1x)),matrix((-1*d2hdv2),nrow(d2hdv2),ncol(d2hdv2))))  
-       BB<-rbind(cbind(matrix((t(x)%*%W1x),nrow(t(x)%*%W1x),ncol(t(x)%*%W1x)),matrix((t(x)%*%W1z),nrow(t(x)%*%W1z),ncol(t(x)%*%W1z))),cbind(matrix((t(z)%*%W1x),nrow(t(z)%*%W1x),ncol(t(z)%*%W1x)),matrix((t(z)%*%W1z),nrow(t(z)%*%W1z),ncol(t(z)%*%W1z))))  
+       AA<-rbind(cbind(matrix((t(x)%*%W1x),nrow(t(x)%*%W1x),ncol(t(x)%*%W1x)),matrix((t(x)%*%W1z),nrow(t(x)%*%W1z),ncol(t(x)%*%W1z))),cbind(matrix((t(z)%*%W1x),nrow(t(z)%*%W1x),ncol(t(z)%*%W1x)),matrix((-1*d2hdv2),nrow(d2hdv2),ncol(d2hdv2))))  ## fast  #
+       BB<-rbind(cbind(matrix((t(x)%*%W1x),nrow(t(x)%*%W1x),ncol(t(x)%*%W1x)),matrix((t(x)%*%W1z),nrow(t(x)%*%W1z),ncol(t(x)%*%W1z))),cbind(matrix((t(z)%*%W1x),nrow(t(z)%*%W1x),ncol(t(z)%*%W1x)),matrix((t(z)%*%W1z),nrow(t(z)%*%W1z),ncol(t(z)%*%W1z))))  ## fast  ##
         cc1<-svd(AA/(2*pi))
         logdet1<-sum(log(abs(cc1$d)))
        rl<--2*hlikeli-2*sum(hv)+logdet1 ##-log(2*pi*nrow(AA))
 ##       rl<--2*hlikeli-2*sum(hv)+logdet1-log(2*pi*nrow(AA))
-       pd<- sum(diag(solve(AA) %*% BB))  
+       pd<- sum(diag(solve(AA) %*% BB))  ## fast  ##
        caic<- -2*hlikeli + 2*pd
        sd<- -2*hh + 2*ll_y
        sd<-sum(deviance)
