@@ -1,5 +1,5 @@
 hglmfit_corr <-
-function(formulaMain,DataMain,Offset=NULL,RespDist="gaussian",RespLink="identity",
+function(formulaMain,BinomialDen=NULL,DataMain,Offset=NULL,RespDist="gaussian",RespLink="identity",
 RandDist="gaussian",mord=0,dord=1,spatial=NULL,Neighbor=NULL,Maxiter=200,Iter_mean=5,convergence=10^(-4),
 Init_lam=0.25,Init_rho=0.174,contrasts=NULL){
     mc <- match.call()
@@ -15,6 +15,7 @@ Init_lam=0.25,Init_rho=0.174,contrasts=NULL){
     p<-ncol(x)
     nrand <- length(z)
     q <- rep(0, nrand)
+    if (is.null(BinomialDen)) BinomialDen<-(y+1)/(y+1)
     for (i in 1:nrand) q[i] <- dim(z[[i]])[2]
     z<-zz<-z[[1]]
     if (is.null(spatial)) spatial="IND"
@@ -185,6 +186,7 @@ for(k in 1:Iter_mean) {
     v_h_old<-v_h
     v_h<-v_h-solve(d2hdv2)%*%dhdv
     c_v_h<-sum(abs(as.vector(v_h_old)-as.vector(v_h)))
+    sv_h<-v_h/sqrt(lambda)
     iter_v<-iter_v+1
 ##    }
 ##############################################################
@@ -540,6 +542,21 @@ for(k in 1:Iter_mean) {
     if (dord<=1) like_value<-cbind(m2h,m2pvh,m2pbvh)
     if (dord<=1) colnames(like_value) <- c("-2*h","-2*p_v(h)","-2p_b,v(h)")
 ##    print(like_value)
-    res<-list(namesX,beta_h,se_beta,lam,rho,clam_se,v_h,like_value,hlikeli1,mu,W2,se_lam,se_rho,A)
+    if (RespDist=="gaussian") deviance_residual<-(y-mu)^2
+    if (RespDist=="poisson") {
+       y_zero<-1*(y==0)
+       deviance_residual<-2*y_zero*mu+(1-y_zero)*2*((y+0.00001)*log((y+0.00001)/mu)-(y+0.00001-mu))
+#       y_zero<-1*(y==0)
+#       deviance_residual<-(2*y_zero*mu+(1-y_zero)*2*((y+0.00001)*log((y+0.00001)/mu)-(y+0.00001-mu)))/old_disp_est
+    }
+    if (RespDist=="binomial") {
+       deviance_residual<-2*y*log((y+0.000001)/mu)+2*(BinomialDen-y)*log((BinomialDen-y+0.000001)/(BinomialDen-mu))
+    }
+    if (RespDist=="gamma") deviance_residual<-2*(-log(y/mu)+(y-mu)/mu)
+    if (RespDist=="gaussian") mean_residual<-sign(y-mu)*sqrt(deviance_residual)*sqrt(1/phi)
+    if (RespDist=="poisson") mean_residual<-sign(y-mu)*sqrt(deviance_residual)
+    if (RespDist=="binomial") mean_residual<-sign(y-mu)*sqrt(deviance_residual)
+    if (RespDist=="gamma") mean_residual<-sign(y-mu)*sqrt(deviance_residual)*sqrt(1/phi)
+    res<-list(namesX,beta_h,se_beta,lam,rho,clam_se,v_h,like_value,hlikeli1,mu,W2,se_lam,se_rho,A,mean_residual)
     return(res)
 }
